@@ -2,7 +2,7 @@
 #define VIENNACL_LINALG_DETAIL_ILUT_HPP_
 
 /* =========================================================================
-   Copyright (c) 2010-2015, Institute for Microelectronics,
+   Copyright (c) 2010-2016, Institute for Microelectronics,
                             Institute for Analysis and Scientific Computing,
                             TU Wien.
    Portions of this software are copyright by UChicago Argonne, LLC.
@@ -35,6 +35,7 @@
 
 #include <map>
 
+#include <Rcpp.h>
 namespace viennacl
 {
 namespace linalg
@@ -308,7 +309,7 @@ void precondition(viennacl::compressed_matrix<NumericT> const & A,
         diagonal_U[i] = value;
         if (value <= 0 && value >= 0)
         {
-          std::cerr << "ViennaCL: FATAL ERROR in ILUT(): Diagonal entry computed to zero (" << value << ") in row " << i << "!" << std::endl;
+          Rcpp::Rcerr << "ViennaCL: FATAL ERROR in ILUT(): Diagonal entry computed to zero (" << value << ") in row " << i << "!" << std::endl;
           throw zero_on_diagonal_exception("ILUT zero diagonal!");
         }
       }
@@ -357,9 +358,9 @@ public:
   ilut_precond(MatrixT const & mat, ilut_tag const & tag) : tag_(tag), L_(mat.size1(), mat.size2()), U_(mat.size1(), mat.size2())
   {
     //initialize preconditioner:
-    //std::cout << "Start CPU precond" << std::endl;
+    //Rcpp::Rcout << "Start CPU precond" << std::endl;
     init(mat);
-    //std::cout << "End CPU precond" << std::endl;
+    //Rcpp::Rcout << "End CPU precond" << std::endl;
   }
 
   template<typename VectorT>
@@ -418,9 +419,9 @@ public:
       U_(mat.size1(), mat.size2(), viennacl::traits::context(mat))
   {
     //initialize preconditioner:
-    //std::cout << "Start GPU precond" << std::endl;
+    //Rcpp::Rcout << "Start GPU precond" << std::endl;
     init(mat);
-    //std::cout << "End GPU precond" << std::endl;
+    //Rcpp::Rcout << "End GPU precond" << std::endl;
   }
 
   void apply(viennacl::vector<NumericT> & vec) const
@@ -429,7 +430,7 @@ public:
     {
       if (tag_.use_level_scheduling())
       {
-        //std::cout << "Using multifrontal on GPU..." << std::endl;
+        //Rcpp::Rcout << "Using multifrontal on GPU..." << std::endl;
         detail::level_scheduling_substitute(vec,
                                             multifrontal_L_row_index_arrays_,
                                             multifrontal_L_row_buffers_,
